@@ -32,7 +32,6 @@
 #include <malloc.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <stdbool.h>
 #include <string.h>
 #include <getopt.h>
 #include <fcntl.h>
@@ -95,7 +94,6 @@ enum {
 	VUMETER_STEREO
 };
 
-static snd_pcm_format_t default_format = DEFAULT_FORMAT;
 static char *command;
 static snd_pcm_t *handle;
 static struct {
@@ -470,24 +468,6 @@ static long parse_long(const char *str, int *err)
 	return val;
 }
 
-static void try_to_adjust_default_format_16bit(void)
-{
-	snd_pcm_hw_params_t *params;
-	int err;
-
-	snd_pcm_hw_params_alloca(&params);
-	err = snd_pcm_hw_params_any(handle, params);
-	if (err < 0) {
-		error(_("Broken configuration for this PCM: no configurations available"));
-		prg_exit(EXIT_FAILURE);
-	}
-
-	if (file_type != FORMAT_AU && snd_pcm_hw_params_test_format(handle, params, SND_PCM_FORMAT_S16_LE) == 0)
-		rhwparams.format = default_format = SND_PCM_FORMAT_S16_LE;
-	else if (file_type == FORMAT_AU && snd_pcm_hw_params_test_format(handle, params, SND_PCM_FORMAT_S16_BE) == 0)
-		rhwparams.format = default_format = SND_PCM_FORMAT_S16_BE;
-}
-
 int main(int argc, char *argv[])
 {
 	int duration_or_sample = 0;
@@ -545,10 +525,9 @@ int main(int argc, char *argv[])
 	};
 	char *pcm_name = "default";
 	int tmp, err, c;
-	int do_device_list = 0, do_pcm_list = 0;
+	int do_device_list = 0, do_pcm_list = 0, force_sample_format = 0;
 	snd_pcm_info_t *info;
 	FILE *direction;
-	bool user_set_fmt = false;
 
 #ifdef ENABLE_NLS
 	setlocale(LC_ALL, "");
@@ -583,7 +562,7 @@ int main(int argc, char *argv[])
 	}
 
 	chunk_size = -1;
-	rhwparams.format = default_format;
+	rhwparams.format = DEFAULT_FORMAT;
 	rhwparams.rate = DEFAULT_SPEED;
 	rhwparams.channels = 1;
 
@@ -633,7 +612,7 @@ int main(int argc, char *argv[])
 			}
 			break;
 		case 'f':
-			user_set_fmt = true;
+			force_sample_format = 1;
 			if (strcasecmp(optarg, "cd") == 0 || strcasecmp(optarg, "cdr") == 0) {
 				if (strcasecmp(optarg, "cdr") == 0)
 					rhwparams.format = SND_PCM_FORMAT_S16_BE;
@@ -866,8 +845,13 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	if (!user_set_fmt)
-		try_to_adjust_default_format_16bit();
+	if (!force_sample_format &&
+	    isatty(fileno(stdin)) &&
+	    stream == SND_PCM_STREAM_CAPTURE &&
+	    snd_pcm_format_width(rhwparams.format) <= 8)
+		fprintf(stderr, "Warning: Some sources (like microphones) may produce inaudible results\n"
+				"         with 8-bit sampling. Use '-f' argument to increase resolution\n"
+				"         e.g. '-f S16_LE'.\n");
 
 	chunk_size = 1024;
 	hwparams = rhwparams;
@@ -1089,7 +1073,7 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 	hwparams.channels = channels;
 	switch (TO_CPU_SHORT(f->bit_p_spl, big_endian)) {
 	case 8:
-		if (hwparams.format != default_format &&
+		if (hwparams.format != DEFAULT_FORMAT &&
 		    hwparams.format != SND_PCM_FORMAT_U8)
 			fprintf(stderr, _("Warning: format is changed to U8\n"));
 		hwparams.format = SND_PCM_FORMAT_U8;
@@ -1099,7 +1083,7 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 			native_format = SND_PCM_FORMAT_S16_BE;
 		else
 			native_format = SND_PCM_FORMAT_S16_LE;
-		if (hwparams.format != default_format &&
+		if (hwparams.format != DEFAULT_FORMAT &&
 		    hwparams.format != native_format)
 			fprintf(stderr, _("Warning: format is changed to %s\n"),
 				snd_pcm_format_name(native_format));
@@ -1112,7 +1096,7 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 				native_format = SND_PCM_FORMAT_S24_3BE;
 			else
 				native_format = SND_PCM_FORMAT_S24_3LE;
-			if (hwparams.format != default_format &&
+			if (hwparams.format != DEFAULT_FORMAT &&
 			    hwparams.format != native_format)
 				fprintf(stderr, _("Warning: format is changed to %s\n"),
 					snd_pcm_format_name(native_format));
@@ -1123,7 +1107,7 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 				native_format = SND_PCM_FORMAT_S24_BE;
 			else
 				native_format = SND_PCM_FORMAT_S24_LE;
-			if (hwparams.format != default_format &&
+			if (hwparams.format != DEFAULT_FORMAT &&
 			    hwparams.format != native_format)
 				fprintf(stderr, _("Warning: format is changed to %s\n"),
 					snd_pcm_format_name(native_format));
@@ -1209,19 +1193,19 @@ static int test_au(int fd, void *buffer)
 	pbrec_count = BE_INT(ap->data_size);
 	switch (BE_INT(ap->encoding)) {
 	case AU_FMT_ULAW:
-		if (hwparams.format != default_format &&
+		if (hwparams.format != DEFAULT_FORMAT &&
 		    hwparams.format != SND_PCM_FORMAT_MU_LAW)
 			fprintf(stderr, _("Warning: format is changed to MU_LAW\n"));
 		hwparams.format = SND_PCM_FORMAT_MU_LAW;
 		break;
 	case AU_FMT_LIN8:
-		if (hwparams.format != default_format &&
+		if (hwparams.format != DEFAULT_FORMAT &&
 		    hwparams.format != SND_PCM_FORMAT_U8)
 			fprintf(stderr, _("Warning: format is changed to U8\n"));
 		hwparams.format = SND_PCM_FORMAT_U8;
 		break;
 	case AU_FMT_LIN16:
-		if (hwparams.format != default_format &&
+		if (hwparams.format != DEFAULT_FORMAT &&
 		    hwparams.format != SND_PCM_FORMAT_S16_BE)
 			fprintf(stderr, _("Warning: format is changed to S16_BE\n"));
 		hwparams.format = SND_PCM_FORMAT_S16_BE;
@@ -1774,10 +1758,12 @@ static void print_vu_meter_stereo(int *perc, int *maxperc)
 		if (c)
 			memset(line + bar_length + 6 + 1, '#', p);
 		else
-			memset(line + bar_length - p - 1, '#', p);
-		p = maxperc[c] * bar_length / 100;
-		if (p > bar_length)
-			p = bar_length;
+			memset(line + bar_length - p, '#', p);
+		p = maxperc[c] * bar_length / 100 - 1;
+		if (p < 0)
+			p = 0;
+		else if (p >= bar_length)
+			p = bar_length - 1;
 		if (c)
 			line[bar_length + 6 + 1 + p] = '+';
 		else
@@ -1844,9 +1830,10 @@ static void compute_max_peak(u_char *data, size_t samples)
 				sval = le16toh(*valp);
 			else
 				sval = be16toh(*valp);
-			sval = abs(sval) ^ mask;
-			if (max_peak[c] < sval)
-				max_peak[c] = sval;
+			sval ^= mask;
+			val = abs(sval);
+			if (max_peak[c] < val)
+				max_peak[c] = val;
 			valp++;
 			if (vumeter == VUMETER_STEREO)
 				c = !c;
@@ -1864,11 +1851,12 @@ static void compute_max_peak(u_char *data, size_t samples)
 			} else {
 				val = (valp[0]<<16) | (valp[1]<<8) | valp[2];
 			}
+			val ^= mask;
 			/* Correct signed bit in 32-bit value */
 			if (val & (1<<(bits_per_sample-1))) {
 				val |= 0xff<<24;	/* Negate upper bits too */
 			}
-			val = abs(val) ^ mask;
+			val = abs(val);
 			if (max_peak[c] < val)
 				max_peak[c] = val;
 			valp += 3;
@@ -1887,7 +1875,11 @@ static void compute_max_peak(u_char *data, size_t samples)
 				val = le32toh(*valp);
 			else
 				val = be32toh(*valp);
-			val = abs(val) ^ mask;
+			val ^= mask;
+			if (val == 0x80000000U)
+				val = 0x7fffffff;
+			else
+				val = abs(val);
 			if (max_peak[c] < val)
 				max_peak[c] = val;
 			valp++;
@@ -1908,6 +1900,8 @@ static void compute_max_peak(u_char *data, size_t samples)
 		max = 0x7fffffff;
 
 	for (c = 0; c < ichans; c++) {
+		if (max_peak[c] > max)
+			max_peak[c] = max;
 		if (bits_per_sample > 16)
 			perc[c] = max_peak[c] / (max / 100);
 		else
@@ -2236,7 +2230,7 @@ static ssize_t pcm_read(u_char *data, size_t rcount)
 		}
 	}
 abort:
-	return rcount;
+	return result > rcount ? rcount : result;
 }
 
 static ssize_t pcm_readv(u_char **data, unsigned int channels, size_t rcount)
@@ -2390,7 +2384,7 @@ static void voc_play(int fd, int ofs, char *name)
 			prg_exit(EXIT_FAILURE);
 		}
 	}
-	hwparams.format = default_format;
+	hwparams.format = DEFAULT_FORMAT;
 	hwparams.channels = 1;
 	hwparams.rate = DEFAULT_SPEED;
 	set_params();
@@ -3294,7 +3288,7 @@ static void capture(char *orig_name)
 			}
 			count -= c;
 			rest -= c;
-			fdcount += c;
+			fdcount += save;
 		}
 
 		/* re-enable SIGUSR1 signal */
