@@ -287,6 +287,58 @@ static bool tplg_is_attribute_valid_value(snd_config_t *valid_values, const char
 	return false;
 }
 
+#if SND_LIB_VER(1, 2, 5) < SND_LIB_VERSION
+static int pre_process_find_variable(snd_config_t **dst, const char *str, snd_config_t *config);
+
+/* check is attribute value belongs in the set of valid values after expanding the valid values */
+static bool tplg_is_attribute_valid_expanded_value(struct tplg_pre_processor *tplg_pp,
+						   snd_config_t *valid_values, int value)
+{
+	snd_config_iterator_t i, next;
+	snd_config_t *n, *var, *conf_defines;
+	int ret;
+
+	ret = snd_config_search(tplg_pp->input_cfg, "Define", &conf_defines);
+	if (ret < 0)
+		return false;
+
+	snd_config_for_each(i, next, valid_values) {
+		const char *s, *id;
+		long v;
+
+		n = snd_config_iterator_entry(i);
+		if (snd_config_get_id(n, &id) < 0)
+			continue;
+
+		if (snd_config_get_string(n, &s) < 0)
+			continue;
+
+		if (s[0] != '$')
+			continue;
+
+		/* find the variable definition */
+		ret = pre_process_find_variable(&var, s + 1, conf_defines);
+		if (ret < 0) {
+			SNDERR("No definition for variable %s\n", s + 1);
+			return false;
+		}
+
+		if (snd_config_get_integer(var, &v) < 0) {
+			SNDERR("Invalid definition for %s\n", s);
+			snd_config_delete(var);
+			return false;
+		}
+
+		snd_config_delete(var);
+
+		if (value == v)
+			return true;
+	}
+
+	return false;
+}
+#endif
+
 /* check if attribute value passes the min/max value constraints */
 static bool tplg_object_is_attribute_min_max_valid(snd_config_t *attr, snd_config_t *obj_attr,
 						   bool min_check)
@@ -372,10 +424,10 @@ static bool tplg_object_is_attribute_min_max_valid(snd_config_t *attr, snd_confi
 
 /* check for min/max and valid value constraints */
 static bool tplg_object_is_attribute_valid(struct tplg_pre_processor *tplg_pp,
-					   snd_config_t *attr, snd_config_t *object)
+					   snd_config_t *attr, snd_config_t *obj_attr)
 {
 	snd_config_iterator_t i, next;
-	snd_config_t *valid, *obj_attr, *n;
+	snd_config_t *valid, *n;
 	snd_config_type_t type;
 	const char *attr_name, *obj_value;
 	int ret;
@@ -383,11 +435,6 @@ static bool tplg_object_is_attribute_valid(struct tplg_pre_processor *tplg_pp,
 	if (snd_config_get_id(attr, &attr_name) < 0)
 		return false;
 
-	ret = snd_config_search(object, attr_name, &obj_attr);
-	if (ret < 0) {
-		SNDERR("attr %s not found \n", attr_name);
-		return false;
-	}
 	type = snd_config_get_type(obj_attr);
 
 	/* check if attribute has valid values */
@@ -421,6 +468,21 @@ static bool tplg_object_is_attribute_valid(struct tplg_pre_processor *tplg_pp,
 			}
 		}
 		return true;
+#if SND_LIB_VER(1, 2, 5) < SND_LIB_VERSION
+	case SND_CONFIG_TYPE_INTEGER:
+	{
+		long v;
+
+		if (snd_config_get_integer(obj_attr, &v) < 0)
+			return false;
+
+		if (!tplg_is_attribute_valid_expanded_value(tplg_pp, valid, v)) {
+			tplg_attribute_print_valid_values(valid, attr_name);
+			return false;
+		}
+		return true;
+	}
+#endif
 	default:
 		break;
 	}
@@ -1014,8 +1076,7 @@ const struct build_function_map object_build_map[] = {
 	 &hwcfg_config},
 	{"Base", "fe_dai", "dai", &tplg_build_fe_dai_object, NULL, &fe_dai_config},
 	{"Base", "route", "SectionGraph", &tplg_build_dapm_route_object, NULL, NULL},
-	{"Widget", "buffer", "SectionWidget", &tplg_build_generic_object,
-	 tplg_update_buffer_auto_attr, &widget_config},
+	{"Widget", "buffer", "SectionWidget", &tplg_build_generic_object, NULL, &widget_config},
 	{"Widget", "", "SectionWidget", &tplg_build_generic_object, NULL, &widget_config},
 	{"Control", "mixer", "SectionControlMixer", &tplg_build_mixer_control, NULL,
 	 &mixer_control_config},
@@ -1087,10 +1148,13 @@ static int tplg_object_copy_and_add_param(struct tplg_pre_processor *tplg_pp,
 					  snd_config_t *attr_cfg,
 					  snd_config_t *search_config)
 {
-	snd_config_t *attr, *new;
+	snd_config_iterator_t first = snd_config_iterator_first(obj);
+	snd_config_t *attr, *new, *first_cfg;
 	const char *id, *search_id;
 	int ret;
 
+	first_cfg = snd_config_iterator_entry(first);
+
 	if (snd_config_get_id(attr_cfg, &id) < 0)
 		return 0;
 
@@ -1108,10 +1172,19 @@ static int tplg_object_copy_and_add_param(struct tplg_pre_processor *tplg_pp,
 		return ret;
 	}
 
-	ret = snd_config_add(obj, new);
-	if (ret < 0) {
-		snd_config_delete(new);
-		SNDERR("error adding attribute '%s' value to %s\n", id, search_id);
+	if (first_cfg) {
+		/* prepend the new config */
+		ret = snd_config_add_before(first_cfg, new);
+		if (ret < 0) {
+			snd_config_delete(new);
+			SNDERR("error prepending attribute '%s' value to %s\n", id, search_id);
+		}
+	} else {
+		ret = snd_config_add(obj, new);
+		if (ret < 0) {
+			snd_config_delete(new);
+			SNDERR("error adding attribute '%s' value to %s\n", id, search_id);
+		}
 	}
 
 	return ret;
@@ -1174,7 +1247,7 @@ static int tplg_object_update(struct tplg_pre_processor *tplg_pp, snd_config_t *
 		ret = snd_config_search(obj_cfg, id, &attr);
 		if (ret < 0)
 			goto class;
-		goto validate;
+		continue;
 class:
 		/* search for attributes value in class */
 		ret = tplg_object_copy_and_add_param(tplg_pp, obj_cfg, n, class_cfg);
@@ -1188,7 +1261,7 @@ class:
 		}
 		else if (ret < 0)
 			return ret;
-		goto validate;
+		continue;
 parent:
 		/* search for attribute value in parent */
 		if (!parent)
@@ -1204,7 +1277,7 @@ parent:
 			goto parent_object;
 		else if (ret < 0)
 			return ret;
-		goto validate;
+		continue;
 parent_object:
 		if (!parent)
 			goto parent_class;
@@ -1219,7 +1292,7 @@ parent_object:
 			goto parent_class;
 		else if (ret < 0)
 			return ret;
-		goto validate;
+		continue;
 parent_class:
 		if (!parent)
 			goto check;
@@ -1238,16 +1311,12 @@ parent_class:
 			goto check;
 		else if (ret < 0)
 			return ret;
-		goto validate;
+		continue;
 check:
 		if (tplg_class_is_attribute_mandatory(id, class_cfg)) {
 			SNDERR("Mandatory attribute %s not set for class %s\n", id, class_name);
 			return -EINVAL;
 		}
-		continue;
-validate:
-		if (!tplg_object_is_attribute_valid(tplg_pp, n, obj_cfg))
-			return -EINVAL;
 	}
 
 	return 0;
@@ -1488,12 +1557,68 @@ snd_config_t *tplg_object_get_instance_config(struct tplg_pre_processor *tplg_pp
 	return snd_config_iterator_entry(first);
 }
 
+#if SND_LIB_VER(1, 2, 5) < SND_LIB_VERSION
+static int pre_process_find_variable(snd_config_t **dst, const char *str, snd_config_t *config)
+{
+	snd_config_iterator_t i, next;
+
+	snd_config_for_each(i, next, config) {
+		snd_config_t *n;
+		const char *id;
+
+		n = snd_config_iterator_entry(i);
+		if (snd_config_get_id(n, &id) < 0)
+			continue;
+
+		if (strcmp(id, str))
+			continue;
+
+		/* found definition, copy config */
+		return snd_config_copy(dst, n);
+	}
+
+	return -EINVAL;
+}
+
+static int
+pre_process_object_variables_expand_fcn(snd_config_t **dst, const char *str, void *private_data)
+{
+
+	struct tplg_pre_processor *tplg_pp = private_data;
+	snd_config_t *object_cfg = tplg_pp->current_obj_cfg;
+	snd_config_t *conf_defines;
+	const char *object_id;
+	int ret;
+
+	ret = snd_config_search(tplg_pp->input_cfg, "Define", &conf_defines);
+	if (ret < 0)
+		return 0;
+
+	/* find variable from global definitions first */
+	ret = pre_process_find_variable(dst, str, conf_defines);
+	if (ret >= 0)
+		return ret;
+
+	if (snd_config_get_id(object_cfg, &object_id) < 0)
+		return -EINVAL;
+
+	/* find variable from object attribute values if not found in global definitions */
+	ret = pre_process_find_variable(dst, str, object_cfg);
+	if (ret < 0)
+		SNDERR("Failed to find definition for attribute %s in '%s' object\n",
+		       str, object_id);
+
+	return ret;
+}
+#endif
+
 /* build object config and its child objects recursively */
 static int tplg_build_object(struct tplg_pre_processor *tplg_pp, snd_config_t *new_obj,
 			      snd_config_t *parent)
 {
 	snd_config_t *obj_local, *class_cfg;
 	const struct build_function_map *map;
+	snd_config_iterator_t i, next;
 	build_func builder;
 	update_auto_attr_func auto_attr_updater;
 	const char *id, *class_id;
@@ -1527,6 +1652,63 @@ static int tplg_build_object(struct tplg_pre_processor *tplg_pp, snd_config_t *n
 		return ret;
 	}
 
+#if SND_LIB_VER(1, 2, 5) < SND_LIB_VERSION
+	tplg_pp_config_debug(tplg_pp, obj_local);
+
+	/* expand all non-compound type child configs in object */
+	snd_config_for_each(i, next, obj_local) {
+		snd_config_t *n, *new, *class_attr;
+		const char *id, *s;
+		char *attr_config_name;
+
+		n = snd_config_iterator_entry(i);
+
+		if (snd_config_get_type(n) == SND_CONFIG_TYPE_COMPOUND)
+			continue;
+
+		if (snd_config_get_id(n, &id) < 0)
+			continue;
+
+		if (snd_config_get_string(n, &s) < 0)
+			continue;
+
+		if (*s != '$')
+			goto validate;
+
+		tplg_pp->current_obj_cfg = obj_local;
+
+		/* expand config */
+		ret = snd_config_evaluate_string(&new, s, pre_process_object_variables_expand_fcn,
+						 tplg_pp);
+		if (ret < 0) {
+			SNDERR("Failed to evaluate attributes %s in %s\n", id, class_id);
+			return ret;
+		}
+
+		snd_config_set_id(new, id);
+
+		ret = snd_config_merge(n, new, true);
+		if (ret < 0)
+			return ret;
+validate:
+		/* validate attribute value */
+		snd_config_get_id(n, &id);
+		attr_config_name = tplg_snprintf("DefineAttribute.%s", id);
+		if (!attr_config_name)
+			return -ENOMEM;
+
+		ret = snd_config_search(class_cfg, attr_config_name, &class_attr);
+		free(attr_config_name);
+		if (ret < 0)
+			continue;
+
+		if (!tplg_object_is_attribute_valid(tplg_pp, class_attr, n)) {
+			SNDERR("Failed to validate attribute %s in %s\n", id, class_id);
+			return -EINVAL;
+		}
+	}
+#endif
+
 	/* construct object name using class constructor */
 	ret = tplg_construct_object_name(tplg_pp, obj_local, class_cfg);
 	if (ret < 0) {
